@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#![cfg_attr(not(target_os = "optee"), no_std)]
+#![feature(c_size_t)]
 #![allow(non_camel_case_types, non_snake_case)]
 
 pub use tee_api::*;
