@@ -16,7 +16,7 @@
 // under the License.
 
 use super::*;
-use libc::*;
+use core::ffi::*;
 
 extern "C" {
     /// tee_invoke_supp_plugin() - invoke a tee-supplicant's plugin
@@ -32,8 +32,8 @@ extern "C" {
         cmd: u32,
         sub_cmd: u32,
         buf: *mut c_char,
-        len: u32,
-        outlen: *mut u32,
+        len: usize,
+        outlen: *mut usize,
     ) -> TEE_Result; 
 
 }
