@@ -15,6 +15,32 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#![cfg_attr(not(target_os = "optee"), no_std)]
+#![cfg_attr(not(target_os = "optee"), feature(error_in_core))]
+
+// Requires `alloc`.
+#[macro_use]
+extern crate alloc;
+
+#[cfg(not(target_os = "optee"))]
+use libc_alloc::LibcAlloc;
+
+#[cfg(not(target_os = "optee"))]
+#[global_allocator]
+static ALLOCATOR: LibcAlloc = LibcAlloc;
+
+#[cfg(not(target_os = "optee"))]
+use core::panic::PanicInfo;
+#[cfg(not(target_os = "optee"))]
+use optee_utee_sys as raw;
+
+#[cfg(all(not(target_os = "optee"), not(feature = "no_panic_handler")))]
+#[panic_handler]
+fn panic(_info: &PanicInfo) -> ! {
+    unsafe { raw::TEE_Panic(0); }
+    loop {}
+}
+
 pub use self::error::{Error, ErrorKind, Result};
 pub use self::object::*;
 pub use self::crypto_op::*;
