@@ -18,10 +18,14 @@
 use crate::{Error, Result};
 use bitflags::bitflags;
 use optee_utee_sys as raw;
-use std::{marker, mem, ptr};
+use core::{marker, mem, ptr};
+#[cfg(not(target_os = "optee"))]
+use alloc::boxed::Box;
+#[cfg(not(target_os = "optee"))]
+use alloc::vec::Vec;
 
 /// A general attribute (buffer or value) that can be used to populate an object or to specify
-/// opeation parameters.
+/// operation parameters.
 pub struct Attribute {
     raw: raw::TEE_Attribute,
 }
@@ -95,7 +99,7 @@ impl<'attrref> AttributeMemref<'attrref> {
                 &mut res.raw,
                 id as u32,
                 buffer.as_ptr() as *mut _,
-                buffer.len() as u32,
+                buffer.len(),
             );
         }
         res
@@ -128,7 +132,7 @@ impl AttributeValue {
     /// # Parameters
     ///
     /// 1) `id`: The AttributeId[AttributeId] is an identifier of the attribute to populate.
-    /// 2) `a`, `b`: u32 values to assign to the members of the a value attribute.
+    /// 2) `a`, `b`: u32 values to assign to the members of the value attribute.
     ///
     /// # Example
     ///
@@ -155,7 +159,7 @@ pub struct ObjectInfo {
 
 // Since raw struct is not implemented Copy attribute yet, every item in raw struct needs a function to extract.
 impl ObjectInfo {
-    /// Return an [ObjectInfo](ObjectInfo) struct based on the raw structrure `TEE_ObjectInfo`.
+    /// Return an [ObjectInfo](ObjectInfo) struct based on the raw structure `TEE_ObjectInfo`.
     /// The raw structure contains following fields:
     ///
     /// 1) `objectType`: The parameter represents one of the
@@ -179,12 +183,12 @@ impl ObjectInfo {
         Self { raw }
     }
 
-    /// Return the `dataSize` field of the raw structrure `TEE_ObjectInfo`.
+    /// Return the `dataSize` field of the raw structure `TEE_ObjectInfo`.
     pub fn data_size(&self) -> usize {
         self.raw.dataSize as usize
     }
 
-    /// Return the `objectSize` field of the raw structrure `TEE_ObjectInfo`.
+    /// Return the `objectSize` field of the raw structure `TEE_ObjectInfo`.
     pub fn object_size(&self) -> usize {
         self.raw.objectSize as usize
     }
@@ -220,6 +224,10 @@ impl ObjectHandle {
         unsafe { *(self.raw) }
     }
 
+    fn is_null(&self) -> bool {
+        self.raw.is_null()
+    }
+
     fn from_raw(raw: *mut raw::TEE_ObjectHandle) -> ObjectHandle {
         Self { raw }
     }
@@ -240,16 +248,16 @@ impl ObjectHandle {
     }
 
     fn ref_attribute(&self, id: AttributeId, buffer: &mut [u8]) -> Result<usize> {
-        let mut size = buffer.len() as u32;
+        let mut size = buffer.len();
         match unsafe {
             raw::TEE_GetObjectBufferAttribute(
                 self.handle(),
                 id as u32,
                 buffer as *mut _ as _,
-                &mut size as _,
+                &mut size,
             )
         } {
-            raw::TEE_SUCCESS => Ok(size as usize),
+            raw::TEE_SUCCESS => Ok(size),
             code => Err(Error::from_raw_error(code)),
         }
     }
@@ -312,13 +320,13 @@ bitflags! {
     pub struct UsageFlag: u32 {
         /// The object [Attribute](Attribute) can be extracted.
         const EXTRACTABLE = 0x00000001;
-        /// The object can be used for encrytpion.
+        /// The object can be used for encryption.
         const ENCRYPT = 0x00000002;
-        /// The object can be used for decrption.
+        /// The object can be used for decryption.
         const DECRYPT = 0x00000004;
         /// The object can be used for mac operation.
         const MAC = 0x00000008;
-        /// The ojbect can be used for signature.
+        /// The object can be used for signature.
         const SIGN = 0x00000010;
         /// The object can be used for verification of a signature.
         const VERIFY = 0x00000020;
@@ -387,7 +395,7 @@ pub enum AttributeId {
     DsaPrivateValue = 0xC0000231,
     /// Diffie-Hellman prime number: `p`
     DhPrime = 0xD0001032,
-    /// Diffie-Hellman sub prime number: `q`
+    /// Diffie-Hellman subprime number: `q`
     DhSubprime = 0xD0001132,
     /// Diffie-Hellman base: `g`
     DhBase = 0xD0001232,
@@ -398,11 +406,22 @@ pub enum AttributeId {
     /// Diffie-Hellman public value: `x`
     DhPrivateValue = 0xC0000232,
     RsaOaepLabel = 0xD0000930,
+    RsaOaepMgf1Hash = 0xD0000931,
     RsaPssSaltLength = 0xF0000A30,
+    /// ECC public value: `x`
     EccPublicValueX = 0xD0000141,
+    /// ECC public value: `y`
     EccPublicValueY = 0xD0000241,
     /// ECC private value: `d`
     EccPrivateValue = 0xC0000341,
+    /// Ed25519 public value
+    Ed25519PublicValue = 0xD0000743,
+    /// Ed25519 private value
+    Ed25519PrivateValue = 0xC0000843,
+    /// X25519 public value
+    X25519PublicValue = 0xD0000944,
+    /// X25519 private value
+    X25519PrivateValue = 0xC0000A44,
     /// ECC Curve algorithm
     EccCurve = 0xF0000441,
     BitProtected = (1 << 28),
@@ -430,7 +449,7 @@ pub enum TransientObjectType {
     HmacSha384 = 0xA0000005,
     /// Between 256 and 1024 bits, multiple of 8 bits
     HmacSha512 = 0xA0000006,
-    /// The number of bits in the modulus. 256, 512, 768, 1024, 1536 and 2048 bit keys SHALL be supported.
+    /// The number of bits in the modulus. 256, 512, 768, 1024, 1536 and 2048-bit keys SHALL be supported.
     /// Support for other key sizes including bigger key sizes is
     /// implementation-dependent. Minimum key size is 256 bits
     RsaPublicKey = 0xA0000030,
@@ -462,6 +481,18 @@ pub enum TransientObjectType {
     /// one of the ECC curves defined in Table 6-14 with "generic" equal to
     /// "Y" is supported. SHALL be same value as for ECDH public key size
     EcdhKeypair = 0xA1000042,
+    /// 256 bits. Conditional: Available only if TEE_ECC_CURVE_25519
+    /// defined in Table 6-14 is supported.
+    Ed25519PublicKey = 0xA0000043,
+    /// 256 bits. Conditional: Available only if TEE_ECC_CURVE_25519
+    /// defined in Table 6-14 is supported.
+    Ed25519Keypair = 0xA1000043,
+    /// 256 bits. Conditional: Available only if TEE_ECC_CURVE_25519
+    /// defined in Table 6-14 is supported.
+    X25519PublicKey = 0xA0000044,
+    /// 256 bits. Conditional: Available only if TEE_ECC_CURVE_25519
+    /// defined in Table 6-14 is supported.
+    X25519Keypair = 0xA1000044,
     /// Multiple of 8 bits, up to 4096 bits. This type is intended for secret
     /// data that has been derived from a key derivation scheme.
     GenericSecret = 0xA0000000,
@@ -471,7 +502,7 @@ pub enum TransientObjectType {
     Data = 0xA00000BF,
 }
 
-/// A trait for an object (trasient or persistent) to return its handle.
+/// A trait for an object (transient or persistent) to return its handle.
 pub trait ObjHandle {
     /// Return the handle of an object.
     fn handle(&self) -> raw::TEE_ObjectHandle;
@@ -490,6 +521,15 @@ impl TransientObject {
         Self(ObjectHandle::from_raw(ptr::null_mut()))
     }
 
+    /// Check if current object is created with null handle.
+    ///
+    /// # See Also
+    ///
+    /// - [Self::null_object](Self::null_object).
+    pub fn is_null_object(&self) -> bool {
+        self.0.is_null()
+    }
+
     /// Allocate an uninitialized [TransientObject](TransientObject), i.e. a container for attributes.
     ///
     /// As allocated, the object is uninitialized.
@@ -626,7 +666,7 @@ impl TransientObject {
     ///
     /// # Parameters
     ///
-    /// 1) `obj_usage`: New object usage, an OR comination of one or more of the [UsageFlag](UsageFlag).
+    /// 1) `obj_usage`: New object usage, an OR combination of one or more of the [UsageFlag](UsageFlag).
     ///
     /// # Example
     ///
@@ -756,7 +796,7 @@ impl TransientObject {
     ///
     /// # Errors
     ///
-    /// 1) `CorruptObject`: If the persistentd` object is corrupt. The object handle is closed.
+    /// 1) `CorruptObject`: If the persistent` object is corrupt. The object handle is closed.
     /// 2) `StorageNotAvailable`: If the [PersistentObject](PersistentObject) is stored in a storage area which is
     ///    currently inaccessible.
     ///
@@ -845,7 +885,7 @@ impl Drop for TransientObject {
             if self.0.raw != ptr::null_mut() {
                 raw::TEE_FreeTransientObject(self.0.handle());
             }
-            Box::from_raw(self.0.raw);
+            drop(Box::from_raw(self.0.raw));
         }
     }
 }
@@ -860,7 +900,7 @@ impl PersistentObject {
     ///
     /// # Parameters
     ///
-    /// 1) `storage_id`: The storaget to use which is defined in
+    /// 1) `storage_id`: The storage to use which is defined in
     ///    [ObjectStorageConstants](ObjectStorageConstants).
     /// 2) `object_id`: The object identifier. Note that this buffer cannot reside in shared memory.
     /// 3) `flags`: The [DataFlag](DataFlag) which determine the settings under which the object is opened.
@@ -908,7 +948,7 @@ impl PersistentObject {
             raw::TEE_OpenPersistentObject(
                 storage_id as u32,
                 object_id.as_ptr() as _,
-                object_id.len() as u32,
+                object_id.len(),
                 flags.bits(),
                 raw_handle as *mut _,
             )
@@ -919,7 +959,7 @@ impl PersistentObject {
             }
             code => {
                 unsafe {
-                    Box::from_raw(raw_handle);
+                    drop(Box::from_raw(raw_handle));
                 }
                 Err(Error::from_raw_error(code))
             }
@@ -930,7 +970,7 @@ impl PersistentObject {
     ///
     /// # Parameters
     ///
-    /// 1) `storage_id`: The storaget to use which is defined in
+    /// 1) `storage_id`: The storage to use which is defined in
     ///    [ObjectStorageConstants](ObjectStorageConstants).
     /// 2) `object_id`: The object identifier. Note that this buffer cannot reside in shared memory.
     /// 3) `flags`: The [DataFlag](DataFlag) which determine the settings under which the object is opened.
@@ -993,11 +1033,11 @@ impl PersistentObject {
             raw::TEE_CreatePersistentObject(
                 storage_id as u32,
                 object_id.as_ptr() as _,
-                object_id.len() as u32,
+                object_id.len(),
                 flags.bits(),
                 attributes,
                 initial_data.as_ptr() as _,
-                initial_data.len() as u32,
+                initial_data.len(),
                 raw_handle as *mut _,
             )
         } {
@@ -1007,7 +1047,7 @@ impl PersistentObject {
             }
             code => {
                 unsafe {
-                    Box::from_raw(raw_handle);
+                    drop(Box::from_raw(raw_handle));
                 }
                 Err(Error::from_raw_error(code))
             }
@@ -1050,7 +1090,7 @@ impl PersistentObject {
         match unsafe { raw::TEE_CloseAndDeletePersistentObject1(self.0.handle()) } {
             raw::TEE_SUCCESS => {
                 unsafe {
-                    Box::from_raw(self.0.raw);
+                    drop(Box::from_raw(self.0.raw));
                 }
                 return Ok(());
             }
@@ -1099,7 +1139,7 @@ impl PersistentObject {
             raw::TEE_RenamePersistentObject(
                 self.0.handle(),
                 new_object_id.as_ptr() as _,
-                new_object_id.len() as u32,
+                new_object_id.len(),
             )
         } {
             raw::TEE_SUCCESS => Ok(()),
@@ -1154,7 +1194,7 @@ impl PersistentObject {
         self.0.value_attribute(id)
     }
 
-    /// Read requested size from the data stream assoicate with the object into the buffer.
+    /// Read requested size from the data stream associate with the object into the buffer.
     ///
     /// # Parameters
     ///
@@ -1190,21 +1230,21 @@ impl PersistentObject {
     /// 2) If the Implementation detects any other error associated with this function which is not
     ///    explicitly associated with a defined return code for this function.
     pub fn read(&self, buf: &mut [u8]) -> Result<u32> {
-        let mut count: u32 = 0;
+        let mut count: usize = 0;
         match unsafe {
             raw::TEE_ReadObjectData(
                 self.handle(),
                 buf.as_mut_ptr() as _,
-                buf.len() as u32,
+                buf.len(),
                 &mut count,
             )
         } {
-            raw::TEE_SUCCESS => Ok(count),
+            raw::TEE_SUCCESS => Ok(count as u32),
             code => Err(Error::from_raw_error(code)),
         }
     }
 
-    /// Write the passed in buffer data into from the data stream assoicate with the object.
+    /// Write the passed in buffer data into from the data stream associate with the object.
     ///
     /// # Parameters
     ///
@@ -1245,7 +1285,7 @@ impl PersistentObject {
     ///    explicitly associated with a defined return code for this function.
     pub fn write(&mut self, buf: &[u8]) -> Result<()> {
         match unsafe {
-            raw::TEE_WriteObjectData(self.handle(), buf.as_ptr() as _, buf.len() as u32)
+            raw::TEE_WriteObjectData(self.handle(), buf.as_ptr() as _, buf.len())
         } {
             raw::TEE_SUCCESS => Ok(()),
             code => Err(Error::from_raw_error(code)),
@@ -1286,7 +1326,7 @@ impl PersistentObject {
     /// 2) If the Implementation detects any other error associated with this function which is not
     ///    explicitly associated with a defined return code for this function.
     pub fn truncate(&self, size: u32) -> Result<()> {
-        match unsafe { raw::TEE_TruncateObjectData(self.handle(), size) } {
+        match unsafe { raw::TEE_TruncateObjectData(self.handle(), size as usize) } {
             raw::TEE_SUCCESS => Ok(()),
             code => Err(Error::from_raw_error(code)),
         }
@@ -1355,7 +1395,7 @@ impl Drop for PersistentObject {
             if self.0.raw != Box::into_raw(Box::new(ptr::null_mut())) {
                 raw::TEE_CloseObject(self.0.handle());
             }
-            Box::from_raw(self.0.raw);
+            drop(Box::from_raw(self.0.raw));
         }
     }
 }
@@ -1376,7 +1416,7 @@ impl ObjectEnumHandle {
             raw::TEE_SUCCESS => Ok(Self { raw: raw_handle }),
             code => {
                 unsafe {
-                    Box::from_raw(raw_handle);
+                    drop(Box::from_raw(raw_handle));
                 }
                 Err(Error::from_raw_error(code))
             }
@@ -1407,7 +1447,7 @@ impl ObjectEnumHandle {
         object_info: &mut ObjectInfo,
         object_id: &mut [u8],
     ) -> Result<u32> {
-        let mut object_id_len: u32 = 0;
+        let mut object_id_len: usize = 0;
         match unsafe {
             raw::TEE_GetNextPersistentObject(
                 *self.raw,
@@ -1416,7 +1456,7 @@ impl ObjectEnumHandle {
                 &mut object_id_len,
             )
         } {
-            raw::TEE_SUCCESS => Ok(object_id_len),
+            raw::TEE_SUCCESS => Ok(object_id_len as u32),
             code => Err(Error::from_raw_error(code)),
         }
     }
@@ -1432,7 +1472,7 @@ impl Drop for ObjectEnumHandle {
     fn drop(&mut self) {
         unsafe {
             raw::TEE_FreePersistentObjectEnumerator(*self.raw);
-            Box::from_raw(self.raw);
+            drop(Box::from_raw(self.raw));
         }
     }
 }
